@@ -1,11 +1,11 @@
 #!/bin/bash -e
 
 # This scrip is for building AppImage
-# Please run this scrip in docker image: ubuntu:16.04
-# E.g: docker run --rm -v `git rev-parse --show-toplevel`:/build ubuntu:16.04 /build/.github/workflows/build_appimage.sh
+# Please run this scrip in docker image: ubuntu:18.04
+# E.g: docker run --rm -v `git rev-parse --show-toplevel`:/build ubuntu:18.04 /build/.github/workflows/build_appimage.sh
 # If you need keep store build cache in docker volume, just like:
 #   $ docker volume create qbee-cache
-#   $ docker run --rm -v `git rev-parse --show-toplevel`:/build -v qbee-cache:/var/cache/apt -v qbee-cache:/usr/src ubuntu:16.04 /build/.github/workflows/build_appimage.sh
+#   $ docker run --rm -v `git rev-parse --show-toplevel`:/build -v qbee-cache:/var/cache/apt -v qbee-cache:/usr/src ubuntu:18.04 /build/.github/workflows/build_appimage.sh
 # Artifacts will copy to the same directory.
 
 set -o pipefail
@@ -14,6 +14,7 @@ set -o pipefail
 export QT_VER_PREFIX="6"
 export LIBTORRENT_BRANCH="RC_2_0"
 
+rm -f /etc/apt/sources.list.d/*.list*
 # Ubuntu mirror for local building
 if [ x"${USE_CHINA_MIRROR}" = x1 ]; then
   source /etc/os-release
@@ -33,11 +34,13 @@ rm -f /etc/apt/apt.conf.d/*
 echo 'Binary::apt::APT::Keep-Downloaded-Packages "true";' >/etc/apt/apt.conf.d/01keep-debs
 echo -e 'Acquire::https::Verify-Peer "false";\nAcquire::https::Verify-Host "false";' >/etc/apt/apt.conf.d/99-trust-https
 
+# Since cmake 3.23.0 CMAKE_INSTALL_LIBDIR will force set to lib/<multiarch-tuple> on Debian
+echo '/usr/local/lib/x86_64-linux-gnu' >/etc/ld.so.conf.d/x86_64-linux-gnu-local.conf
+
 apt update
 apt install -y software-properties-common apt-transport-https
-apt-add-repository -y ppa:savoury1/backports
-apt-add-repository -y ppa:savoury1/toolchain
-add-apt-repository -y ppa:savoury1/display
+apt-add-repository -yn ppa:savoury1/backports
+add-apt-repository -yn ppa:savoury1/display
 
 if [ x"${USE_CHINA_MIRROR}" = x1 ]; then
   sed -i 's@http://ppa.launchpad.net@https://launchpad.proxy.ustclug.org@' /etc/apt/sources.list.d/*.list
@@ -56,7 +59,6 @@ apt install -y \
   libxcb1-dev \
   libicu-dev \
   libgtk2.0-dev \
-  g++-8 \
   build-essential \
   libgl1-mesa-dev \
   libfontconfig1-dev \
@@ -84,8 +86,8 @@ apt install -y \
   libxkbcommon-dev \
   libxkbcommon-x11-dev \
   libwayland-dev \
-  libwayland-egl-backend-dev
-# libgtk-3-dev
+  libwayland-egl-backend-dev \
+  g++-8
 
 apt autoremove --purge -y
 # make gcc-8 as default gcc
@@ -192,7 +194,10 @@ rm -fr CMakeCache.txt CMakeFiles
   -no-eglfs \
   -no-feature-testlib \
   -no-feature-vnc \
-  -feature-optimize_full
+  -feature-optimize_full \
+  -nomake examples \
+  -nomake tests
+cat config.summary
 cmake --build . --parallel
 cmake --install .
 export QT_BASE_DIR="$(ls -rd /usr/local/Qt-* | head -1)"
@@ -216,6 +221,7 @@ fi
 cd "/usr/src/qttools-${qt_ver}"
 rm -fr CMakeCache.txt
 "${QT_BASE_DIR}/bin/qt-configure-module" .
+cat config.summary
 cmake --build . --parallel
 cmake --install .
 
@@ -228,6 +234,7 @@ fi
 cd "/usr/src/qtwayland-${qt_ver}"
 rm -fr CMakeCache.txt
 "${QT_BASE_DIR}/bin/qt-configure-module" .
+cat config.summary
 cmake --build . --parallel
 cmake --install .
 
@@ -267,17 +274,25 @@ fi
 
 # build libtorrent-rasterbar
 echo "libtorrent-rasterbar branch: ${LIBTORRENT_BRANCH}"
+libtorrent_git_url="https://github.com/arvidn/libtorrent.git"
+if [ x"${USE_CHINA_MIRROR}" = x1 ]; then
+  libtorrent_git_url="https://ghproxy.com/${libtorrent_git_url}"
+fi
 if [ ! -d "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/" ]; then
-  libtorrent_git_url="https://github.com/arvidn/libtorrent.git"
-  if [ x"${USE_CHINA_MIRROR}" = x1 ]; then
-    libtorrent_git_url="https://ghproxy.com/${libtorrent_git_url}"
-  fi
   retry git clone --depth 1 --recursive --shallow-submodules --branch "${LIBTORRENT_BRANCH}" \
     "${libtorrent_git_url}" \
     "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
 fi
 cd "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
-git pull
+if ! git pull; then
+  # if pull failed, retry clone the repository.
+  cd /
+  rm -fr "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
+  retry git clone --depth 1 --recursive --shallow-submodules --branch "${LIBTORRENT_BRANCH}" \
+    "${libtorrent_git_url}" \
+    "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
+  cd "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
+fi
 rm -fr build/CMakeCache.txt
 cmake \
   -B build \
