@@ -320,6 +320,8 @@ prepare_qt() {
     -no-feature-testlib \
     -no-feature-animation \
     -feature-optimize_full \
+    -nomake examples \
+    -nomake tests \
     ${QT_BASE_EXTRA_CONF} \
     -device-option "CROSS_COMPILE=${CROSS_HOST}-" \
     -- \
@@ -328,6 +330,7 @@ prepare_qt() {
     -DCMAKE_C_COMPILER="${CROSS_HOST}-gcc" \
     -DCMAKE_SYSROOT="${CROSS_PREFIX}" \
     -DCMAKE_CXX_COMPILER="${CROSS_HOST}-g++"
+  cat config.summary
   cmake --build . --parallel
   cmake --install .
   export QT_BASE_DIR="${CROSS_PREFIX}/opt/qt"
@@ -337,17 +340,25 @@ prepare_qt() {
 
 prepare_libtorrent() {
   echo "libtorrent-rasterbar branch: ${LIBTORRENT_BRANCH}"
+  libtorrent_git_url="https://github.com/arvidn/libtorrent.git"
+  if [ x"${USE_CHINA_MIRROR}" = x1 ]; then
+    libtorrent_git_url="https://ghproxy.com/${libtorrent_git_url}"
+  fi
   if [ ! -d "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/" ]; then
-    libtorrent_git_url="https://github.com/arvidn/libtorrent.git"
-    if [ x"${USE_CHINA_MIRROR}" = x1 ]; then
-      libtorrent_git_url="https://ghproxy.com/${libtorrent_git_url}"
-    fi
     retry git clone --depth 1 --recursive --shallow-submodules --branch "${LIBTORRENT_BRANCH}" \
       "${libtorrent_git_url}" \
       "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
   fi
   cd "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
-  git pull
+  if ! git pull; then
+    # if pull failed, retry clone the repository.
+    cd /
+    rm -fr "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
+    retry git clone --depth 1 --recursive --shallow-submodules --branch "${LIBTORRENT_BRANCH}" \
+      "${libtorrent_git_url}" \
+      "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
+    cd "/usr/src/libtorrent-rasterbar-${LIBTORRENT_BRANCH}/"
+  fi
   rm -fr build/CMakeCache.txt
   # TODO: solve mingw build
   if [ x"${TARGET_HOST}" = xWindows ]; then
