@@ -90,6 +90,8 @@ PeerListWidget::PeerListWidget(PropertiesWidget *parent)
     setEditTriggers(QAbstractItemView::NoEditTriggers);
     setSelectionMode(QAbstractItemView::ExtendedSelection);
     header()->setStretchLastSection(false);
+    header()->setTextElideMode(Qt::ElideRight);
+
     // List Model
     m_listModel = new QStandardItemModel(0, PeerListColumns::COL_COUNT, this);
     m_listModel->setHeaderData(PeerListColumns::COUNTRY, Qt::Horizontal, tr("Country/Region")); // Country flag column
@@ -261,44 +263,54 @@ void PeerListWidget::updatePeerCountryResolutionState()
 
 void PeerListWidget::showPeerListMenu(const QPoint &)
 {
-    BitTorrent::Torrent *const torrent = m_properties->getCurrentTorrent();
+    BitTorrent::Torrent *torrent = m_properties->getCurrentTorrent();
     if (!torrent) return;
 
-    QMenu *menu = new QMenu(this);
+    auto *menu = new QMenu(this);
     menu->setAttribute(Qt::WA_DeleteOnClose);
+    menu->setToolTipsVisible(true);
 
-    // Add Peer Action
-    // Do not allow user to add peers in a private torrent
-    if (!torrent->isQueued() && !torrent->isChecking() && !torrent->isPrivate())
+    QAction *addNewPeer = menu->addAction(UIThemeManager::instance()->getIcon("user-group-new"), tr("Add peers...")
+        , this, [this, torrent]()
     {
-        menu->addAction(UIThemeManager::instance()->getIcon("user-group-new"), tr("Add a new peer...")
-            , this, [this, torrent]()
+        const QVector<BitTorrent::PeerAddress> peersList = PeersAdditionDialog::askForPeers(this);
+        const int peerCount = std::count_if(peersList.cbegin(), peersList.cend(), [torrent](const BitTorrent::PeerAddress &peer)
         {
-            const QVector<BitTorrent::PeerAddress> peersList = PeersAdditionDialog::askForPeers(this);
-            const int peerCount = std::count_if(peersList.cbegin(), peersList.cend(), [torrent](const BitTorrent::PeerAddress &peer)
-            {
-                return torrent->connectPeer(peer);
-            });
-            if (peerCount < peersList.length())
-                QMessageBox::information(this, tr("Adding peers"), tr("Some peers cannot be added. Check the Log for details."));
-            else if (peerCount > 0)
-                QMessageBox::information(this, tr("Adding peers"), tr("Peers are added to this torrent."));
+            return torrent->connectPeer(peer);
         });
-    }
+        if (peerCount < peersList.length())
+            QMessageBox::information(this, tr("Adding peers"), tr("Some peers cannot be added. Check the Log for details."));
+        else if (peerCount > 0)
+            QMessageBox::information(this, tr("Adding peers"), tr("Peers are added to this torrent."));
+    });
+    QAction *copyPeers = menu->addAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Copy IP:port")
+        , this, &PeerListWidget::copySelectedPeers);
+    menu->addSeparator();
+    QAction *banPeers = menu->addAction(UIThemeManager::instance()->getIcon("user-group-delete"), tr("Ban peer permanently")
+        , this, &PeerListWidget::banSelectedPeers);
 
-    if (!selectionModel()->selectedRows().isEmpty())
+    // disable actions
+    const auto disableAction = [](QAction *action, const QString &tooltip)
     {
-        menu->addAction(UIThemeManager::instance()->getIcon("edit-copy"), tr("Copy IP:port")
-            , this, &PeerListWidget::copySelectedPeers);
-        menu->addSeparator();
-        menu->addAction(UIThemeManager::instance()->getIcon("user-group-delete"), tr("Ban peer permanently")
-            , this, &PeerListWidget::banSelectedPeers);
+        action->setEnabled(false);
+        action->setToolTip(tooltip);
+    };
+
+    if (torrent->isPrivate())
+        disableAction(addNewPeer, tr("Cannot add peers to a private torrent"));
+    else if (torrent->isChecking())
+        disableAction(addNewPeer, tr("Cannot add peers when the torrent is checking"));
+    else if (torrent->isQueued())
+        disableAction(addNewPeer, tr("Cannot add peers when the torrent is queued"));
+
+    if (selectionModel()->selectedRows().isEmpty())
+    {
+        const QString tooltip = tr("No peer was selected");
+        disableAction(copyPeers, tooltip);
+        disableAction(banPeers, tooltip);
     }
 
-    if (menu->isEmpty())
-        delete menu;
-    else
-        menu->popup(QCursor::pos());
+    menu->popup(QCursor::pos());
 }
 
 void PeerListWidget::banSelectedPeers()
@@ -462,7 +474,7 @@ void PeerListWidget::updatePeer(const BitTorrent::Torrent *torrent, const BitTor
     setModelData(row, PeerListColumns::CONNECTION, peer.connectionType(), peer.connectionType());
     setModelData(row, PeerListColumns::FLAGS, peer.flags(), peer.flags(), {}, peer.flagsDescription());
     const QString client = peer.client().toHtmlEscaped();
-    setModelData(row, PeerListColumns::CLIENT, client, client);
+    setModelData(row, PeerListColumns::CLIENT, client, client, {}, client);
     const QString peerId = peer.peerId().toHtmlEscaped();
     setModelData(row, PeerListColumns::PEERID, peerId, peerId);
     setModelData(row, PeerListColumns::PROGRESS, (Utils::String::fromDouble(peer.progress() * 100, 1) + '%'), peer.progress(), intDataTextAlignment);
@@ -476,9 +488,11 @@ void PeerListWidget::updatePeer(const BitTorrent::Torrent *torrent, const BitTor
     setModelData(row, PeerListColumns::TOT_UP, totalUp, peer.totalUpload(), intDataTextAlignment);
     setModelData(row, PeerListColumns::RELEVANCE, (Utils::String::fromDouble(peer.relevance() * 100, 1) + '%'), peer.relevance(), intDataTextAlignment);
 
-    const QStringList downloadingFiles {torrent->info().filesForPiece(peer.downloadingPieceIndex())};
+    const QStringList downloadingFiles {torrent->hasMetadata()
+                ? torrent->info().filesForPiece(peer.downloadingPieceIndex())
+                : QStringList()};
     const QString downloadingFilesDisplayValue = downloadingFiles.join(';');
-    setModelData(row, PeerListColumns::DOWNLOADING_PIECE, downloadingFilesDisplayValue, downloadingFilesDisplayValue, {}, downloadingFiles.join('\n'));
+    setModelData(row, PeerListColumns::DOWNLOADING_PIECE, downloadingFilesDisplayValue, downloadingFilesDisplayValue, {}, downloadingFiles.join(QLatin1Char('\n')));
 
     if (m_resolver)
         m_resolver->resolve(peerEndpoint.address.ip);
@@ -519,15 +533,9 @@ void PeerListWidget::wheelEvent(QWheelEvent *event)
     {
         // Shift + scroll = horizontal scroll
         event->accept();
-
-#if (QT_VERSION >= QT_VERSION_CHECK(5, 14, 0))
         QWheelEvent scrollHEvent(event->position(), event->globalPosition()
             , event->pixelDelta(), event->angleDelta().transposed(), event->buttons()
             , event->modifiers(), event->phase(), event->inverted(), event->source());
-#else
-        QWheelEvent scrollHEvent(event->pos(), event->globalPos()
-            , event->delta(), event->buttons(), event->modifiers(), Qt::Horizontal);
-#endif
         QTreeView::wheelEvent(&scrollHEvent);
         return;
     }
